function [bestF bestT bestGain] = bestAttribute(data,labels,binary)
%Javier Hernandez Rivera, MIT

%Computes the conditional entropies of an objective class given all the
%features in the dataset

%Find the different categories of the data
cat = unique(labels);

%Initialize variables
total = [];
cont = 1;

%Compute conditional entropy for all the features
for pos = 1:size(data,2)
    values = unique(data(:,pos));
    values = sort(values);
    if binary(pos) == 0%length(values)>2 %Non binary variable
        
        thresholds = obtainTresholds(data(:,pos)', labels');        
        for i = 1:length(thresholds)
            %Threshold the data
            features = zeros(size(data(:,pos)));
            features(find(data(:,pos)>=thresholds(i))) = 1;
            features(find(data(:,pos)<thresholds(i))) = 0;
            %Compute entropies
            res = computeEntropies(labels, features, sprintf('%d_%g',pos,thresholds(i)));
            %Save information
            total(cont,1) = res;
            total(cont,2) = thresholds(i);
            total(cont,3) = pos;
            cont = cont + 1;
        end
    else %Binary variable
        %Compute entropies
        res = computeEntropies(labels, data(:,pos), sprintf('%d',pos));
        %Save information
        total(cont,1) = res;
        total(cont,2) = values(end);
        total(cont,3) = pos;
        cont = cont + 1;
    end
end

%Compute information gain of each split
x = length(find(labels==cat(1))) / length(labels);
gain = entropy(x) - total(:,1);

%Find best split
[v p] = max(gain);

fprintf('Best Information Gain (%g): F%g_%g\n\n', v, total(p,3), total(p,2));

%Save information
bestF = total(p,3);
bestT = total(p,2);
bestGain = v;
