function curr_node = buildTree(data, labels, fid, features, binary, level)
%Javier Hernandez Rivera, MIT

%find the different categories of the data
cat = unique(labels);


fprintf('==================================\n');
fprintf('Level %d\n', level);
fprintf('==================================\n');

if isempty(data) || length(cat)==1
    %Leaf case
    class = mode(labels); % associate with the most common class

    %save information for printing
    new_node  = sprintf('%d (%d)',class,round(rand()*20000));
    fprintf(fid,'\t"%s" [label="%d"];\n',new_node, class);
    curr_node = new_node;

else %Internal node
    %Obtain best feature to split
    [bestF bestT bestGain] = bestAttribute(data, labels, binary);
    curr_node =  sprintf('%s\\nIG = %.3f (%d)',features{bestF}, bestGain, round(rand()*20000));
    fprintf(fid,'\t"%s" [label="%s"];\n',curr_node, sprintf('%s\\nIG = %.3f',features{bestF}, bestGain));

    %Left node
    idx = find(data(:,bestF)<bestT);

    if length(unique(data(idx,bestF)))==1 % Remove feature
        remove = bestF;
    else
        remove = size(data,2)+1;        
    end
    ndata = data(idx,[1:size(data,2)]~=remove);
    nfeatures = {features{[1:size(data,2)]~=remove}};
    nbinary = binary([1:size(data,2)]~=remove);

    fprintf('-----------------------\n');
    fprintf('Left Node at Level %d\n',level);
    fprintf('-----------------------\n');

    ant_node = buildTree(ndata, labels(idx), fid, nfeatures, nbinary, level + 1);
    fprintf(fid,'\t"%s" -> "%s" [label="%s" color="black"];\n',curr_node, ant_node, sprintf('<%g',bestT));

    %Right node
    idx = find(data(:,bestF)>=bestT);

    if length(unique(data(idx,bestF)))==1 % Remove feature
        remove = bestF;
    else
        remove = size(data,2)+1;        
    end
    ndata = data(idx,[1:size(data,2)]~=remove);
    nfeatures = {features{[1:size(data,2)]~=remove}};
    nbinary = binary([1:size(data,2)]~=remove);

    fprintf('-----------------------\n');
    fprintf('Right Node at Level %d\n',level);
    fprintf('-----------------------\n');
    ant_node = buildTree(ndata, labels(idx), fid, nfeatures, nbinary, level + 1);
    fprintf(fid,'\t"%s" -> "%s" [label="%s" color="black"];\n',curr_node, ant_node, sprintf('>=%g',bestT));
end
